﻿using System;
using System.Collections.Generic;
using System.Drawing;
using System.IO;
using System.Windows.Forms;
using System.Windows.Forms.DataVisualization.Charting;

namespace YGO_DeckAnalyzer
{
    public partial class FormChart : Form
    {
        public FormChart(Dictionary<string, int> list)
        {
            deckTypesList = list;
            this.InitializeComponent();
        }
        Dictionary<string, int> deckTypesList = new Dictionary<string, int>();
        List<string> deckTypes = new List<string>();
        List<int> deckNums = new List<int>();
        private void ShowChart_CombineOthers(Dictionary<string, int> list)
        {
            deckTypes = new List<string>();
            deckNums = new List<int>();
            int total = 0;
            foreach (var kvp in list)
            {
                total += kvp.Value;
            }
            foreach (var kvp in list)
            {
                if (kvp.Key == "其他" || (kvp.Value * 1.00 / total < 0.03))
                {
                    if (deckTypes.Contains("其他"))
                    {
                        deckNums[deckTypes.IndexOf("其他")] += kvp.Value;
                    }
                    else
                    {
                        deckTypes.Add("其他");
                        deckNums.Add(kvp.Value);
                    }
                }
                else
                {
                    deckTypes.Add(kvp.Key);
                    deckNums.Add(kvp.Value);
                }
            }
            cChart.Series[0].Points.DataBindXY(deckTypes, deckNums);
            this.SetBackground();
        }
        private void SetBackground()
        {
            foreach (var point in cChart.Series[0].Points)
            {
                if (File.Exists($"pics\\{point.AxisLabel}.jpg"))
                {
                    point.BackImage = new FileInfo($"pics\\{point.AxisLabel}.jpg").FullName;
                    point.BackImageWrapMode = ChartImageWrapMode.TileFlipX;
                }
                else if (File.Exists($"pics\\{point.AxisLabel}.png"))
                {
                    point.BackImage = new FileInfo($"pics\\{point.AxisLabel}.png").FullName;
                    point.BackImageWrapMode = ChartImageWrapMode.TileFlipX;
                }
            }
        }
        private void ShowChart_SepOthers(Dictionary<string, int> list)
        {
            deckTypes = new List<string>();
            deckNums = new List<int>();
            int total = 0;
            foreach (var kvp in list)
            {
                total += kvp.Value;
            }
            foreach (var kvp in list)
            {
                deckTypes.Add(kvp.Key);
                deckNums.Add(kvp.Value);
            }
            cChart.Series[0].Points.DataBindXY(deckTypes, deckNums);
            this.SetBackground();
        }
        private void FormChart_Load(object sender, EventArgs e)
        {
            this.ShowChart_CombineOthers(deckTypesList);
            cChart.Series[0]["PieLabelStyle"] = "Outside";
            cChart.Series[0]["PieLineColor"] = "Black";
            cChart.Series[0].ChartType = SeriesChartType.Pie;
            if (File.Exists("pics\\bg.jpg"))
            {
                cChart.Series[0].LabelBackColor = Color.Transparent;
                cChart.Series[0].BackSecondaryColor = Color.Transparent;
                cChart.Series[0].ShadowColor = Color.Transparent;
                cChart.Series[0].BorderColor = Color.Transparent;
                cChart.ChartAreas[0].BackColor = Color.Transparent;
                cChart.Legends[0].BackColor = Color.Transparent;
                cChart.BackImage = new FileInfo("pics\\bg.jpg").FullName;
                cChart.BackImageWrapMode = ChartImageWrapMode.Scaled;
            }
            else if (File.Exists("pics\\bg.png"))
            {
                cChart.Series[0].LabelBackColor = Color.Transparent;
                cChart.Series[0].BackSecondaryColor = Color.Transparent;
                cChart.Series[0].ShadowColor = Color.Transparent;
                cChart.Series[0].BorderColor = Color.Transparent;
                cChart.ChartAreas[0].BackColor = Color.Transparent;
                cChart.Legends[0].BackColor = Color.Transparent;
                cChart.BackImage = new FileInfo("pics\\bg.png").FullName;
                cChart.BackImageWrapMode = ChartImageWrapMode.Scaled;
            }

        }

        private void TrackBar1_Scroll(object sender, EventArgs e)
        {
            try
            {
                cChart.Series[0].ChartType = (SeriesChartType)((int)tbChartType.Value);
            }
            catch { }
        }

        private void BtnSepOthers_Click(object sender, EventArgs e)
        {
            this.ShowChart_SepOthers(deckTypesList);
        }

        private void BtnCombineOthers_Click(object sender, EventArgs e)
        {
            this.ShowChart_CombineOthers(deckTypesList);
        }

        private void MenuSave_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                if (sfdImage.FilterIndex == 1)
                {
                    cChart.SaveImage(sfdImage.FileName, ChartImageFormat.Jpeg);
                }
                else
                {
                    cChart.SaveImage(sfdImage.FileName, ChartImageFormat.Png);
                }
            }
        }

        private void MenuCopy_Click(object sender, EventArgs e)
        {
            MemoryStream ms = new MemoryStream();
            this.cChart.SaveImage(ms, ChartImageFormat.Jpeg);
            Bitmap m = new Bitmap(ms);
            //复制到粘贴板
            Clipboard.SetImage(m);
        }

        private void MenuCopyHighRes_Click(object sender, EventArgs e)
        {
            Size s = cChart.Size;
            cChart.Size = new Size(4096, 3000);
            MemoryStream ms = new MemoryStream();
            this.cChart.SaveImage(ms, ChartImageFormat.Jpeg);
            Bitmap m = new Bitmap(ms);
            //复制到粘贴板
            Clipboard.SetImage(m);
            cChart.Size = s;
        }

        private void MenuHighResSave_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                Size s = cChart.Size;
                cChart.Size = new Size(4096, 3000);
                if (sfdImage.FilterIndex == 1)
                {
                    cChart.SaveImage(sfdImage.FileName, ChartImageFormat.Jpeg);
                }
                else
                {
                    cChart.SaveImage(sfdImage.FileName, ChartImageFormat.Png);
                }
                cChart.Size = s;
            }
        }

        private void BtnSetColor_Click(object sender, EventArgs e)
        {
            if (fontSetFont.ShowDialog() == DialogResult.OK)
            {
                cChart.Series[0].Font = fontSetFont.Font;
                cChart.Legends[0].Font = fontSetFont.Font;
            }
        }

        private void BtnColor_Click(object sender, EventArgs e)
        {
            if (colorSetColor.ShowDialog() == DialogResult.OK)
            {
                cChart.Series[0].LabelForeColor = colorSetColor.Color;
                cChart.Legends[0].ForeColor = colorSetColor.Color;
            }
        }

        private void BtnSetLineColor_Click(object sender, EventArgs e)
        {
            if (colorSetLineColor.ShowDialog() == DialogResult.OK)
            {
                string r = colorSetLineColor.Color.R.ToString("X2");
                string g = colorSetLineColor.Color.G.ToString("X2");
                string b = colorSetLineColor.Color.B.ToString("X2");
                cChart.Series[0]["PieLineColor"] = $"0x{r}{g}{b}";
            }
        }

        private void CChart_Click(object sender, EventArgs e)
        {

        }

        private void TrackImageScale_Scroll(object sender, EventArgs e)
        {
            int add = trackImageScale.Value;
            cChart.ChartAreas[0].InnerPlotPosition = new ElementPosition(10 + add, 10 + add, 80 - add * 2, 80 - add * 2);
        }

        private void menuSaveExtraHighRes_Click(object sender, EventArgs e)
        {
            if (sfdImage.ShowDialog() == DialogResult.OK)
            {
                Size s = cChart.Size;
                cChart.Size = new Size(8192, 6000);
                if (sfdImage.FilterIndex == 1)
                {
                    cChart.SaveImage(sfdImage.FileName, ChartImageFormat.Jpeg);
                }
                else
                {
                    cChart.SaveImage(sfdImage.FileName, ChartImageFormat.Png);
                }
                cChart.Size = s;
            }
        }

        private void menuCopyExtraHighRes_Click(object sender, EventArgs e)
        {
            Size s = cChart.Size;
            cChart.Size = new Size(8192, 6000);
            MemoryStream ms = new MemoryStream();
            this.cChart.SaveImage(ms, ChartImageFormat.Jpeg);
            Bitmap m = new Bitmap(ms);
            //复制到粘贴板
            Clipboard.SetImage(m);
            cChart.Size = s;
        }
    }
}
